import javax.swing.*;
import java.awt.*;
import java.util.List;
import java.util.*;
import java.awt.geom.*;

class Shapes {
	public static void main(String[] args) {
		Circle c1 = new Circle();
		Circle c2 = new Circle();
		c2.moveHorizontal(50);
		Square s1 = new Square();
		s1.changeColor("yellow");
		s1.moveVertical(50);
		Square s2 = new Square();
		s2.changeColor("green");
		s2.moveVertical(50);
		s2.moveHorizontal(-50);
	}
}

class Circle {

	int diameter;

	int xPosition;

	int yPosition;

	String color;

	Circle() {
		diameter = 30;
		xPosition = 20;
		yPosition = 60;
		color = "blue";
		draw();
	}

	void moveHorizontal(int distance) {
		erase();
		xPosition += distance;
		draw();
	}

	void moveVertical(int distance) {
		erase();
		yPosition += distance;
		draw();
	}

	void changeSize(int newDiameter) {
		erase();
		diameter = newDiameter;
		draw();
	}

	void changeColor(String newColor) {
		color = newColor;
		draw();
	}

	void draw() {
		Canvas canvas = Canvas.getCanvas();
		canvas.draw(this, color, new Ellipse2D.Double(xPosition, yPosition,
				diameter, diameter));
		canvas.wait(1000);
	}

	void erase() {
		Canvas canvas = Canvas.getCanvas();
		canvas.erase(this);
	}
}

class Square {

	int size;

	int xPosition;

	int yPosition;

	String color;

	Square() {
		size = 30;
		xPosition = 60;
		yPosition = 50;
		color = "red";
		draw();
	}

	void moveHorizontal(int distance) {
		erase();
		xPosition += distance;
		draw();
	}

	void moveVertical(int distance) {
		erase();
		yPosition += distance;
		draw();
	}

	void changeSize(int newSize) {
		erase();
		size = newSize;
		draw();
	}

	void changeColor(String newColor) {
		color = newColor;
		draw();
	}

	void draw() {
		Canvas canvas = Canvas.getCanvas();
		canvas.draw(this, color,
				new Rectangle(xPosition, yPosition, size, size));
		canvas.wait(1000);

	}

	void erase() {
		Canvas canvas = Canvas.getCanvas();
		canvas.erase(this);
	}
}

class Canvas {

	static Canvas canvasSingleton;

	static {
		canvasSingleton = new Canvas("Shapes", 500, 500, Color.white);
		canvasSingleton.setVisible(true);
	}

	static Canvas getCanvas() {
		return canvasSingleton;
	}

	// ----- instance part -----

	JFrame frame;

	CanvasPane canvas;

	Graphics2D graphic;

	Color backgroundColour;

	Image canvasImage;

	List objects;

	HashMap shapes;

	Canvas(String title, int width, int height, Color bgColour) {
		frame = new JFrame();
		canvas = new CanvasPane();
		frame.setContentPane(canvas);
		frame.setTitle(title);
		canvas.setPreferredSize(new Dimension(width, height));
		backgroundColour = bgColour;
		frame.pack();
		objects = new ArrayList();
		shapes = new HashMap();
	}

	void setVisible(boolean visible) {
		if (graphic == null) {
			// first time: instantiate the offscreen image and fill it with
			// the background colour
			Dimension size = canvas.getSize();
			canvasImage = canvas.createImage(size.width, size.height);
			graphic = (Graphics2D) canvasImage.getGraphics();
			graphic.setColor(backgroundColour);
			graphic.fillRect(0, 0, size.width, size.height);
			graphic.setColor(Color.black);
		}
		frame.setVisible(visible);
	}

	void draw(Object referenceObject, String color, Shape shape) {
		objects.remove(referenceObject); // just in case it was already there
		objects.add(referenceObject); // add at the end
		shapes.put(referenceObject, new ShapeDescription(shape, color));
		redraw();
	}

	void erase(Object referenceObject) {
		objects.remove(referenceObject); // just in case it was already there
		shapes.remove(referenceObject);
		redraw();
	}

	void setForegroundColor(String colorString) {
		if (colorString.equals("red"))
			graphic.setColor(Color.red);
		else if (colorString.equals("black"))
			graphic.setColor(Color.black);
		else if (colorString.equals("blue"))
			graphic.setColor(Color.blue);
		else if (colorString.equals("yellow"))
			graphic.setColor(Color.yellow);
		else if (colorString.equals("green"))
			graphic.setColor(Color.green);
		else if (colorString.equals("magenta"))
			graphic.setColor(Color.magenta);
		else if (colorString.equals("white"))
			graphic.setColor(Color.white);
		else
			graphic.setColor(Color.black);
	}

	void wait(int milliseconds) {
		try {
			Thread.sleep(milliseconds);
		} catch (Exception e) {
			// ignoring exception at the moment
		}
	}

	void redraw() {
		erase();
		for (Iterator i = objects.iterator(); i.hasNext();) {
			((ShapeDescription) shapes.get(i.next())).draw(graphic);
		}
		canvas.repaint();
	}

	void erase() {
		Color original = graphic.getColor();
		graphic.setColor(backgroundColour);
		Dimension size = canvas.getSize();
		graphic.fill(new Rectangle(0, 0, size.width, size.height));
		graphic.setColor(original);
	}

	private class CanvasPane extends JPanel {
		public void paint(Graphics g) {
			g.drawImage(canvasImage, 0, 0, null);
		}
	}

	private class ShapeDescription {

		private Shape shape;

		private String colorString;

		ShapeDescription(Shape shape, String color) {
			this.shape = shape;
			colorString = color;
		}

		void draw(Graphics2D graphic) {
			setForegroundColor(colorString);
			graphic.fill(shape);
		}
	}

}
